@@ -1,4 +1,5 @@
 from collections import Counter, defaultdict, deque
+from collections.abc import Sequence
 from functools import partial, wraps
 from heapq import merge
 from itertools import (
@@ -17,9 +18,9 @@ from itertools import (
 )
 from operator import itemgetter, lt, gt, sub
 from sys import maxsize, version_info
-from collections.abc import Sequence
+from time import monotonic
 
-from .recipes import consume, flatten, take
+from .recipes import consume, flatten, powerset, take
 
 __all__ = [
     'adjacent',
@@ -54,6 +55,7 @@ __all__ = [
     'numeric_range',
     'one',
     'padded',
+    'partitions',
     'peekable',
     'replace',
     'rlocate',
@@ -72,6 +74,8 @@ __all__ = [
     'stagger',
     'strip',
     'substrings',
+    'substrings_indexes',
+    'time_limited',
     'unique_to_each',
     'unzip',
     'windowed',
@@ -708,6 +712,40 @@ def substrings(iterable):
             yield seq[i:i + n]
 
 
+def substrings_indexes(seq, reverse=False):
+    """Yield all substrings and their positions in *seq*
+
+    The items yielded will be a tuple of the form ``(substr, i, j)``, where
+    ``substr == seq[i:j]``.
+
+    This function only works for iterables that support slicing, such as
+    ``str`` objects.
+
+    >>> for item in substrings_indexes('more'):
+    ...    print(item)
+    ('m', 0, 1)
+    ('o', 1, 2)
+    ('r', 2, 3)
+    ('e', 3, 4)
+    ('mo', 0, 2)
+    ('or', 1, 3)
+    ('re', 2, 4)
+    ('mor', 0, 3)
+    ('ore', 1, 4)
+    ('more', 0, 4)
+
+    Set *reverse* to ``True`` to yield the same items in the opposite order.
+
+
+    """
+    r = range(1, len(seq) + 1)
+    if reverse:
+        r = reversed(r)
+    return (
+        (seq[i:i + L], i, i + L) for L in r for i in range(len(seq) - L + 1)
+    )
+
+
 class bucket:
     """Wrap *iterable* and return an object that buckets it iterable into
     child iterables based on a *key* function.
@@ -874,7 +912,9 @@ def collapse(iterable, base_type=None, levels=None):
         >>> list(collapse(iterable))
         [1, 2, 3, 4, 5, 6]
 
-    String types are not considered iterable and will not be collapsed.
+    Binary and text strings are not considered iterable and
+    will not be collapsed.
+
     To avoid collapsing other types, specify *base_type*:
 
         >>> iterable = ['ab', ('cd', 'ef'), ['gh', 'ij']]
@@ -893,7 +933,7 @@ def collapse(iterable, base_type=None, levels=None):
     def walk(node, level):
         if (
             ((levels is not None) and (level > levels)) or
-            isinstance(node, str) or
+            isinstance(node, (str, bytes)) or
             ((base_type is not None) and isinstance(node, base_type))
         ):
             yield node
@@ -2304,3 +2344,60 @@ def replace(iterable, pred, substitutes, count=None, window_size=1):
         # yield the first item from the window.
         if w and (w[0] is not _marker):
             yield w[0]
+
+
+def partitions(iterable):
+    """Yield all possible partitions of *iterable*. This yields all the ways
+    to split the iterable, starting with the full iterable, all splits into
+    two pieces, all the splits into three pieces, and so on. Order is preserved
+    and repeated elements are treated as distinct.
+
+    >>> iterable = ['a', 'b', 'c', 'd']
+    >>> for part in partitions(iterable):
+    ...     print(part)
+    [['a', 'b', 'c', 'd']]
+    [['a'], ['b', 'c', 'd']]
+    [['a', 'b'], ['c', 'd']]
+    [['a', 'b', 'c'], ['d']]
+    [['a'], ['b'], ['c', 'd']]
+    [['a'], ['b', 'c'], ['d']]
+    [['a', 'b'], ['c'], ['d']]
+    [['a'], ['b'], ['c'], ['d']]
+
+    This is unrelated to :func:`partition`.
+
+    """
+    sequence = list(iterable)
+    n = len(sequence)
+    for i in powerset(range(1, n)):
+        yield [sequence[i:j] for i, j in zip((0,) + i, i + (n,))]
+
+
+def time_limited(limit_seconds, iterable):
+    """
+    Yield items from *iterable* until *limit_seconds* have passed.
+
+    >>> from time import sleep
+    >>> def generator():
+    ...     yield 1
+    ...     yield 2
+    ...     sleep(0.2)
+    ...     yield 3
+    >>> iterable = generator()
+    >>> list(time_limited(0.1, iterable))
+    [1, 2]
+
+    Note that the time is checked before each item is yielded, and iteration
+    stops if  the time elapsed is greater than *limit_seconds*. If your time
+    limit is 1 second, but it takes 2 seconds to generate the first item from
+    the iterable, the function will run for 2 seconds and not yield anything.
+
+    """
+    if limit_seconds < 0:
+        raise ValueError('limit_seconds must be positive')
+
+    start_time = monotonic()
+    for item in iterable:
+        if monotonic() - start_time > limit_seconds:
+            break
+        yield item
