@@ -16,6 +16,7 @@ from itertools import (
     repeat,
 )
 from operator import add, mul, itemgetter
+from time import sleep
 from unittest import TestCase
 
 import more_itertools as mi
@@ -716,6 +717,70 @@ class SubstringsTests(TestCase):
         self.assertEqual(actual, expected)
 
 
+class SubstringsIndexesTests(TestCase):
+    def test_basic(self):
+        sequence = [x for x in range(4)]
+        actual = list(mi.substrings_indexes(sequence))
+        expected = [
+            ([0], 0, 1),
+            ([1], 1, 2),
+            ([2], 2, 3),
+            ([3], 3, 4),
+            ([0, 1], 0, 2),
+            ([1, 2], 1, 3),
+            ([2, 3], 2, 4),
+            ([0, 1, 2], 0, 3),
+            ([1, 2, 3], 1, 4),
+            ([0, 1, 2, 3], 0, 4),
+        ]
+        self.assertEqual(actual, expected)
+
+    def test_strings(self):
+        sequence = 'abc'
+        actual = list(mi.substrings_indexes(sequence))
+        expected = [
+            ('a', 0, 1),
+            ('b', 1, 2),
+            ('c', 2, 3),
+            ('ab', 0, 2),
+            ('bc', 1, 3),
+            ('abc', 0, 3),
+        ]
+        self.assertEqual(actual, expected)
+
+    def test_empty(self):
+        sequence = []
+        actual = list(mi.substrings_indexes(sequence))
+        expected = []
+        self.assertEqual(actual, expected)
+
+    def test_order(self):
+        sequence = [2, 0, 1]
+        actual = list(mi.substrings_indexes(sequence))
+        expected = [
+            ([2], 0, 1),
+            ([0], 1, 2),
+            ([1], 2, 3),
+            ([2, 0], 0, 2),
+            ([0, 1], 1, 3),
+            ([2, 0, 1], 0, 3),
+        ]
+        self.assertEqual(actual, expected)
+
+    def test_reverse(self):
+        sequence = [2, 0, 1]
+        actual = list(mi.substrings_indexes(sequence, reverse=True))
+        expected = [
+            ([2, 0, 1], 0, 3),
+            ([2, 0], 0, 2),
+            ([0, 1], 1, 3),
+            ([2], 0, 1),
+            ([0], 1, 2),
+            ([1], 2, 3),
+        ]
+        self.assertEqual(actual, expected)
+
+
 class BucketTests(TestCase):
     """Tests for ``bucket()``"""
 
@@ -840,6 +905,12 @@ class TestCollapse(TestCase):
         l = [["s1"], "s2", [["s3"], "s4"], [[["s5"]]]]
         self.assertEqual(list(mi.collapse(l)), ["s1", "s2", "s3", "s4", "s5"])
 
+    def test_collapse_to_bytes(self):
+        l = [[b"s1"], b"s2", [[b"s3"], b"s4"], [[[b"s5"]]]]
+        self.assertEqual(
+            list(mi.collapse(l)), [b"s1", b"s2", b"s3", b"s4", b"s5"]
+        )
+
     def test_collapse_flatten(self):
         l = [[1], [2], [[3], 4], [[[5]]]]
         self.assertEqual(list(mi.collapse(l, levels=1)), list(mi.flatten(l)))
@@ -2305,3 +2376,77 @@ class ReplaceTests(TestCase):
         actual = list(mi.replace(iterable, pred, substitutes))
         expected = ['_', '_', 1, '_', '_', 3, '_', '_']
         self.assertEqual(actual, expected)
+
+
+class PartitionsTest(TestCase):
+    def test_types(self):
+        for iterable in [
+            'abcd',
+            ['a', 'b', 'c', 'd'],
+            ('a', 'b', 'c', 'd'),
+        ]:
+            with self.subTest(iterable=iterable):
+                actual = list(mi.partitions(iterable))
+                expected = [
+                    [['a', 'b', 'c', 'd']],
+                    [['a'], ['b', 'c', 'd']],
+                    [['a', 'b'], ['c', 'd']],
+                    [['a', 'b', 'c'], ['d']],
+                    [['a'], ['b'], ['c', 'd']],
+                    [['a'], ['b', 'c'], ['d']],
+                    [['a', 'b'], ['c'], ['d']],
+                    [['a'], ['b'], ['c'], ['d']]
+                ]
+                self.assertEqual(actual, expected)
+
+    def test_empty(self):
+        iterable = []
+        actual = list(mi.partitions(iterable))
+        expected = [[[]]]
+        self.assertEqual(actual, expected)
+
+    def test_order(self):
+        iterable = iter([3, 2, 1])
+        actual = list(mi.partitions(iterable))
+        expected = [
+            [[3, 2, 1]],
+            [[3], [2, 1]],
+            [[3, 2], [1]],
+            [[3], [2], [1]],
+        ]
+        self.assertEqual(actual, expected)
+
+    def test_duplicates(self):
+        iterable = [1, 1, 1]
+        actual = list(mi.partitions(iterable))
+        expected = [
+            [[1, 1, 1]],
+            [[1], [1, 1]],
+            [[1, 1], [1]],
+            [[1], [1], [1]],
+        ]
+        self.assertEqual(actual, expected)
+
+
+class TimeLimitedTests(TestCase):
+    def test_basic(self):
+        def generator():
+            yield 1
+            yield 2
+            sleep(0.2)
+            yield 3
+
+        iterable = generator()
+        actual = list(mi.time_limited(0.1, iterable))
+        expected = [1, 2]
+        self.assertEqual(actual, expected)
+
+    def test_zero_limit(self):
+        iterable = count()
+        actual = list(mi.time_limited(0, iterable))
+        expected = []
+        self.assertEqual(actual, expected)
+
+    def test_invalid_limit(self):
+        with self.assertRaises(ValueError):
+            list(mi.time_limited(-0.1, count()))
